@@ -1,12 +1,13 @@
 # :coding: utf-8
 
+from __future__ import absolute_import
 import functools
+import logging
 
 import wiz.definition
 import wiz.environ
 import wiz.exception
 import wiz.history
-import wiz.logging
 import wiz.symbol
 
 
@@ -168,7 +169,7 @@ def combine_environ_mapping(package_identifier, mapping1, mapping2):
         This process will stringify all variable values.
 
     """
-    logger = wiz.logging.Logger(__name__ + ".combine_environ_mapping")
+    logger = logging.getLogger(__name__ + ".combine_environ_mapping")
 
     mapping = {}
 
@@ -218,7 +219,7 @@ def combine_command_mapping(package_identifier, mapping1, mapping2):
     :return: Combined command alias mapping.
 
     """
-    logger = wiz.logging.Logger(__name__ + ".combine_command_mapping")
+    logger = logging.getLogger(__name__ + ".combine_command_mapping")
 
     mapping = {}
 
