@@ -1,6 +1,7 @@
 # :coding: utf-8
 
 import copy
+import collections
 import base64
 import hashlib
 
@@ -99,8 +100,8 @@ def test_is_overlapping(
     mocker, mocked_extract_version_ranges, ranges1, ranges2, expected
 ):
     """Indicates whether requirements are overlapping."""
-    req1 = mocker.Mock()
-    req2 = mocker.Mock()
+    req1 = mocker.Mock(extras=set())
+    req2 = mocker.Mock(extras=set())
 
     # Hack due to the impossibility to directly mock an attribute called "name"
     # See: https://bradmontgomery.net/blog/how-world-do-you-mock-name-attribute
@@ -111,17 +112,30 @@ def test_is_overlapping(
     assert wiz.utility.is_overlapping(req1, req2) == expected
 
 
+def test_is_overlapping_with_variants(mocker):
+    """Fail to compare requirement with different variant."""
+    req1 = mocker.Mock(extras={"V1"})
+    req2 = mocker.Mock(extras={"V2"})
+
+    # Hack due to the impossibility to directly mock an attribute called "name"
+    # See: https://bradmontgomery.net/blog/how-world-do-you-mock-name-attribute
+    type(req1).name = mocker.PropertyMock(return_value="foo")
+    type(req2).name = mocker.PropertyMock(return_value="foo")
+
+    assert wiz.utility.is_overlapping(req1, req2) is False
+
+
 def test_is_overlapping_fail(mocker):
     """Fail to compare requirement with different name."""
-    req1 = mocker.Mock()
-    req2 = mocker.Mock()
+    req1 = mocker.Mock(extras=set())
+    req2 = mocker.Mock(extras=set())
 
     # Hack due to the impossibility to directly mock an attribute called "name"
     # See: https://bradmontgomery.net/blog/how-world-do-you-mock-name-attribute
     type(req1).name = mocker.PropertyMock(return_value="foo")
     type(req2).name = mocker.PropertyMock(return_value="bar")
 
-    with pytest.raises(wiz.exception.GraphResolutionError) as error:
+    with pytest.raises(ValueError) as error:
         wiz.utility.is_overlapping(req1, req2)
 
     assert (
@@ -627,6 +641,29 @@ def test_sanitize_requirement_error(mocked_match):
     assert "Requirement 'A' is incompatible with package 'B'" in str(error)
 
 
+@pytest.mark.parametrize("requirements, expected", [
+    ([], {}),
+    ([Requirement("foo"), Requirement("bar")], {"A": 1, "B": 1, "C": 1}),
+    ([Requirement("::foo"), Requirement("B::bar")], {"B": 1}),
+    ([Requirement("foo"), Requirement("B::bar")], {"A": 1, "B": 1}),
+], ids=[
+    "empty",
+    "non-qualified-requirements",
+    "qualified-requirements",
+    "mixed",
+])
+def test_namespace_counter(requirements, expected):
+    """Compute namespace occurrences counter from requirements."""
+    mapping = {
+        "__namespace__": {
+            "foo": ["A"],
+            "bar": ["B", "C"]
+        }
+    }
+    result = wiz.utility.compute_namespace_counter(requirements, mapping)
+    assert result == collections.Counter(expected)
+
+
 @pytest.mark.parametrize("package_data, variant_index, requirement, expected", [
     ({"identifier": "A"}, None, Requirement("A"), True),
     ({"identifier": "B"}, None, Requirement("A"), False),
